# hamiltonian.mod
# Tucker's formulation [Papadimitriou & Steiglitz, Comb Opt p. 309, Eq. (13.3)]

## parameters and sets

# vertices
param n integer, > 0;
set V := 1..n;

# edges
set E within {V,V};
set A := E union {i in V, j in V : (j,i) in E};

# useless parameters (allow reading of format .dat graph format files)
param c{E} default 0;
param cL{E} default 0;
param cU{E} default 0;
param I{E} default 0;

# set of subcycles
param subcycles >= 0, integer, default 0;
set S{1..subcycles};

## decision variables: is the arc in the Hamiltonian path?
var x{A} binary;
var u{V};

## no objective function: this is a decision problem

## constraints

# cycle
subject to successor{i in V} :
  sum{j in V : (i,j) in A} x[i,j] = 1;
subject to predecessor{j in V} :
  sum{i in V : (i,j) in A} x[i,j] = 1;

# no subcycle
subject to tucker{(i,j) in A : i != 1 and j != 1}:
  u[i] - u[j]  <= n*(1 - x[i,j]) - 1